@@ -19,9 +19,9 @@
 
 """Provides the FocusTrackingPresenter for Orca."""
 
-__id__        = "$Id: focus_tracking_presenter.py 2237 2007-04-02 20:42:19Z lmonsanto $"
-__version__   = "$Revision: 2237 $"
-__date__      = "$Date: 2007-04-02 16:42:19 -0400 (Mon, 02 Apr 2007) $"
+__id__        = "$Id: focus_tracking_presenter.py 2374 2007-05-11 17:06:19Z wwalker $"
+__version__   = "$Revision: 2374 $"
+__date__      = "$Date: 2007-05-11 13:06:19 -0400 (Fri, 11 May 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -74,7 +74,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         self._gidleLock      = threading.Lock()
         self.noFocusTimestamp = 0.0
 
-        orca_state.activeScript = None
+        self.setActiveScript(None)
 
         if settings.debugEventQueue:
             self._enqueueEventCount = 0
@@ -265,6 +265,25 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
         return script
 
+    def setActiveScript(self, newScript):
+        """Set the new active script.
+
+        Arguments:
+        - newScript: the new script to be made active.
+        """
+
+        try:
+            orca_state.activeScript.deactivate()
+        except:
+            pass
+
+        orca_state.activeScript = newScript
+
+        try:
+            orca_state.activeScript.activate()
+        except:
+            pass 
+
     def _reclaimScripts(self):
         """Compares the list of known scripts to the list of known apps,
         deleting any scripts as necessary.
@@ -301,13 +320,14 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
     #                                                                      #
     ########################################################################
 
-    def loadAppSettings(self, app):
+    def loadAppSettings(self, script):
         """Load the users application specific settings for an app.
 
         Arguments:
-        - app: the Python app
+        - script: the current active script.
         """
 
+        app = script.app
         settingsPackages = settings.settingsPackages
         moduleName = settings.getScriptModuleName(app)
         module = None
@@ -321,13 +341,24 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 try:
                     debug.println(debug.LEVEL_FINEST,
                                   "Looking for settings at %s.py..." % name)
-                    if self. _knownAppSettings.has_key(name):
-                        reload(self._knownAppSettings[name])
-                    else:
+                    if not self._knownAppSettings.has_key(name):
                         self._knownAppSettings[name] = \
                             __import__(name, globals(), locals(), [''])
+                    reload(self._knownAppSettings[name])
                     debug.println(debug.LEVEL_FINEST,
                                   "...found %s.py" % name)
+
+                    # Setup the user's application specific key bindings.
+                    # (if any).
+                    # 
+                    if hasattr(self._knownAppSettings[name], 
+                               "overrideAppKeyBindings"):
+                        script.overrideAppKeyBindings = \
+                            self._knownAppSettings[name].overrideAppKeyBindings
+                        script.keyBindings = \
+                         self._knownAppSettings[name].overrideAppKeyBindings( \
+                            script, script.keyBindings)
+
                     break
                 except ImportError:
                     debug.println(debug.LEVEL_FINEST,
@@ -387,10 +418,26 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         #
         try:
             if event.source.role == rolenames.ROLE_TOOL_TIP:
-                # Check that it's okay to present tool tips. There
-                # is a bug in GTK versions prior to 2.10.11 that
-                # caused gnome-panel to crash.
-                if not settings.presentToolTips:
+                # Check that it's okay to present tool tips. Always present 
+                # tooltips initiated by the user pressing Control-F1 on the
+                # keyboard.
+                #
+                if isinstance(orca_state.lastInputEvent, \
+                              input_event.KeyboardEvent):
+                    if not orca_state.lastNonModifierKeyEvent.event_string == "F1":
+                        return
+
+                    # Mouse move events don't update orca_state.lastInputEvent
+                    # so it's possible the user accidentally nudged the
+                    # mouse and generated another tooltip event. If the
+                    # current time minus the last keyboard event time is 
+                    # greater than 0.2 seconds, than just ignore this tooltip
+                    # event.
+                    #
+                    currentTime =  time.time()
+                    if (currentTime - orca_state.lastInputEvent.time) > 0.2:
+                        return
+                elif not settings.presentToolTips:
                     return
         except:
             pass
@@ -487,8 +534,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                         # We'll let someone else decide if it's important
                         # to stop speech or not.
                         #speech.stop()
-                        orca_state.activeScript = \
-                            self._getScript(event.source.app)
+                        self.setActiveScript(self._getScript(event.source.app))
                         debug.println(debug.LEVEL_FINE, "ACTIVE SCRIPT: " \
                                       + orca_state.activeScript.name)
 
@@ -505,7 +551,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                         # Load in the application specific settings for the
                         # app for this event (if found).
                         #
-                        appSettings = self.loadAppSettings(event.source.app)
+                        appSettings = self.loadAppSettings( \
+                                                      orca_state.activeScript)
 
                         # Tell BrlTTY which commands we care about.
                         #
@@ -571,16 +618,16 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         event = None
         if isinstance(e, input_event.KeyboardEvent):
             if e.type == atspi.Accessibility.KEY_PRESSED_EVENT:
-                debug.println(debug.LEVEL_FINEST,
+                debug.println(debug.LEVEL_ALL,
                               "----------> QUEUEING KEYPRESS '%s' (%d)"
                               % (e.event_string, e.hw_code))
             elif e.type == atspi.Accessibility.KEY_RELEASED_EVENT:
-                debug.println(debug.LEVEL_FINEST,
+                debug.println(debug.LEVEL_ALL,
                               "----------> QUEUEING KEYRELEASE '%s' (%d)"
                               % (e.event_string, e.hw_code))
             event = e
         elif isinstance(e, input_event.BrailleEvent):
-            debug.println(debug.LEVEL_FINEST,
+            debug.println(debug.LEVEL_ALL,
                           "----------> QUEUEING BRAILLE COMMAND %d" % e.event)
             event = e
         else:
@@ -630,7 +677,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             if (not debug.eventDebugFilter) \
                 or (debug.eventDebugFilter \
                     and debug.eventDebugFilter.match(e.type)):
-                debug.println(debug.LEVEL_FINEST,
+                debug.println(debug.LEVEL_ALL,
                               "---------> QUEUEING EVENT %s" % e.type)
 
         if event:
@@ -641,10 +688,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             if settings.debugEventQueue:
                 debug.println(debug.LEVEL_ALL,
                               "           ...acquired")
-            if settings.debugEventQueue:
                 debug.println(debug.LEVEL_ALL,
                               "           calling queue.put...")
-            if settings.debugEventQueue:
                 debug.println(debug.LEVEL_ALL,
                               "           (full=%s)" % self._eventQueue.full())
             self._eventQueue.put(event)
@@ -695,12 +740,12 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
             if isinstance(event, input_event.KeyboardEvent):
                 if event.type == atspi.Accessibility.KEY_PRESSED_EVENT:
-                    debug.println(debug.LEVEL_FINEST,
+                    debug.println(debug.LEVEL_ALL,
                                   "DEQUEUED KEYPRESS '%s' (%d) <----------" \
                                   % (event.event_string, event.hw_code))
                     pressRelease = "PRESS"
                 elif event.type == atspi.Accessibility.KEY_RELEASED_EVENT:
-                    debug.println(debug.LEVEL_FINEST,
+                    debug.println(debug.LEVEL_ALL,
                                   "DEQUEUED KEYRELEASE '%s' (%d) <----------" \
                                   % (event.event_string, event.hw_code))
                     pressRelease = "RELEASE"
@@ -712,7 +757,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                               "\n^^^^^ PROCESS KEY %s EVENT %s ^^^^^"\
                               % (pressRelease, event.event_string))
             elif isinstance(event, input_event.BrailleEvent):
-                debug.println(debug.LEVEL_FINEST,
+                debug.println(debug.LEVEL_ALL,
                               "DEQUEUED BRAILLE COMMAND %d <----------" \
                               % event.event)
                 debug.println(debug.eventDebugLevel,
@@ -726,7 +771,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 if (not debug.eventDebugFilter) \
                     or (debug.eventDebugFilter \
                         and debug.eventDebugFilter.match(event.type)):
-                    debug.println(debug.LEVEL_FINEST,
+                    debug.println(debug.LEVEL_ALL,
                                   "DEQUEUED EVENT %s <----------" \
                                   % event.type)
                     debug.println(debug.eventDebugLevel,
@@ -860,7 +905,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         self._oldAppSettings = None
         self._defaultScript  = None
 
-        orca_state.activeScript = self._getScript(None)
+        self.setActiveScript(self._getScript(None))
 
         # Tell BrlTTY which commands we care about.
         #
@@ -895,4 +940,4 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         self._oldAppSettings = None
         self._defaultScript  = None
 
-        orca_state.activeScript = None
+        self.setActiveScript(None)
