@@ -33,9 +33,9 @@ created in their own module.  The module defining the Script subclass
 is also required to have a 'getScript(app)' method that returns an
 instance of the Script subclass.  See default.py for an example."""
 
-__id__        = "$Id: script.py 2219 2007-03-28 14:10:49Z richb $"
-__version__   = "$Revision: 2219 $"
-__date__      = "$Date: 2007-03-28 10:10:49 -0400 (Wed, 28 Mar 2007) $"
+__id__        = "$Id: script.py 2365 2007-05-09 20:43:34Z richb $"
+__version__   = "$Revision: 2365 $"
+__date__      = "$Date: 2007-05-09 16:43:34 -0400 (Wed, 09 May 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -181,6 +181,31 @@ class Script:
         """
         return where_am_I.WhereAmI(self)
 
+    def getAppPreferencesGUI(self):
+        """Return a GtkVBox contain the application unique configuration
+        GUI items for the current application.
+        """
+        return None
+
+    def setAppPreferences(self, prefs):
+        """Write out the application specific preferences lines and set the
+        new values.
+
+        Arguments:
+        - prefs: file handle for application preferences.
+        """
+        pass
+
+    def overrideAppKeyBindings(self, script, keyBindings):
+        """Allow for the customization of application specific key bindings.
+
+        Arguments:
+        - script: the application script.
+        - keyBindings: the set of key bindings for this script.
+        """
+
+        return keyBindings
+
     # [[[WDW - There is a circular reference going on somewhere (see
     # bug 333168).  In the presence of this reference, the existence
     # of a __del__ method prevents the garbage collector from
@@ -408,3 +433,11 @@ class Script:
         - obj: the Accessible whose visual appearance changed.
         """
         pass
+
+    def activate(self):
+        """Called when this script is activated."""
+        pass
+
+    def deactivate(self):
+        """Called when this script is deactivated."""
+        pass
