@@ -19,9 +19,9 @@
 
 """Custom script for Evolution."""
 
-__id__        = "$Id: Evolution.py 2302 2007-04-19 17:29:45Z joanied $"
-__version__   = "$Revision: 2302 $"
-__date__      = "$Date: 2007-04-19 13:29:45 -0400 (Thu, 19 Apr 2007) $"
+__id__        = "$Id: Evolution.py 2375 2007-05-11 17:56:08Z richb $"
+__version__   = "$Revision: 2375 $"
+__date__      = "$Date: 2007-05-11 13:56:08 -0400 (Fri, 11 May 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -419,50 +419,64 @@ class Script(default.Script):
         while not done:
             accPanel = htmlPanel.accessible.getChildAtIndex(i)
             panel = atspi.Accessible.makeAccessible(accPanel)
-            accTextObj = panel.accessible.getChildAtIndex(0)
-            textObj = atspi.Accessible.makeAccessible(accTextObj)
-            text = textObj.text
-            if not text:
-                return
-            textObjs.append(textObj)
-            length = text.characterCount
+            if panel != None:
+                accTextObj = panel.accessible.getChildAtIndex(0)
+                textObj = atspi.Accessible.makeAccessible(accTextObj)
+                text = textObj.text
+                if not text:
+                    return
+                textObjs.append(textObj)
+                length = text.characterCount
 
-            while offset <= length:
-                [mystr, start, end] = textObj.text.getTextAtOffset(offset,
-                                                                   mode)
+                while offset <= length:
+                    [mystr, start, end] = textObj.text.getTextAtOffset(offset,
+                                                                       mode)
+                    if len(mystr) != 0:
+                        string += " " + mystr
 
-                if len(mystr) != 0:
-                    string += mystr
+                    if mode == atspi.Accessibility.TEXT_BOUNDARY_LINE_START or \
+                       len(mystr) == 0 or mystr[len(mystr)-1] in '.?!':
+                        endOffset = end
 
-                if mode == atspi.Accessibility.TEXT_BOUNDARY_LINE_START or \
-                   len(mystr) == 0 or mystr[len(mystr)-1] in '.?!':
-                    endOffset = end
+                        string = self.adjustForRepeats(string)
+                        if string.isupper():
+                            voice = settings.voices[settings.UPPERCASE_VOICE]
+                        else:
+                            voice = settings.voices[settings.DEFAULT_VOICE]
 
-                    string = self.adjustForRepeats(string)
-                    if string.isupper():
-                        voice = settings.voices[settings.UPPERCASE_VOICE]
-                    else:
-                        voice = settings.voices[settings.DEFAULT_VOICE]
-
-                    if not textObjs:
-                        textObjs.append(textObj)
-                    yield [speechserver.SayAllContext(textObjs, string,
+                        if not textObjs:
+                            textObjs.append(textObj)
+                        if len(string) != 0:
+                            yield [speechserver.SayAllContext(textObjs, string,
                                                       startOffset, endOffset),
-                           voice]
-                    textObjs = []
-                    string = ""
-                    startOffset = endOffset
+                               voice]
+                        textObjs = []
+                        string = ""
+                        startOffset = endOffset
 
-                if len(mystr) == 0 or end == length:
-                    break
-                else:
-                    offset = end
+                    if len(mystr) == 0 or end == length:
+                        break
+                    else:
+                        offset = end
 
             offset = 0
             i += 1
             if i == total:
                 done = True
 
+        # If there is anything left unspoken, speak it now.
+        # 
+        if len(string) != 0:
+            string = self.adjustForRepeats(string)
+            if string.isupper():
+                voice = settings.voices[settings.UPPERCASE_VOICE]
+            else:
+                voice = settings.voices[settings.DEFAULT_VOICE]
+
+            yield [speechserver.SayAllContext(textObjs, string,
+                                              startOffset, endOffset),
+                   voice]
+
     def __sayAllProgressCallback(self, context, type):
         """Provide feedback during the sayAll operation.
         """
@@ -683,7 +697,7 @@ class Script(default.Script):
             justDeleted = False
             if isinstance(orca_state.lastInputEvent,
                           input_event.KeyboardEvent):
-                string = orca_state.lastInputEvent.event_string
+                string = orca_state.lastNonModifierKeyEvent.event_string
                 if string == "Delete":
                     justDeleted = True
 
@@ -1248,7 +1262,7 @@ class Script(default.Script):
                 orca_state.lastInputEvent.__dict__.has_key("event_string")):
             return False
 
-        lastKey = orca_state.lastInputEvent.event_string
+        lastKey = orca_state.lastNonModifierKeyEvent.event_string
         if lastKey != "Left" and lastKey != "Right":
             return False
 
@@ -1382,7 +1396,7 @@ class Script(default.Script):
         # 
         if isinstance(orca_state.lastInputEvent,
                       input_event.KeyboardEvent):
-            string = orca_state.lastInputEvent.event_string
+            string = orca_state.lastNonModifierKeyEvent.event_string
             if string == "Delete":
                 rolesList = [rolenames.ROLE_TABLE_CELL, \
                              rolenames.ROLE_TREE_TABLE, \
