@@ -23,10 +23,10 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py 2126 2007-03-06 21:35:17Z richb $"
-__version__   = "$Revision: 2126 $"
-__date__      = "$Date: 2007-03-06 16:35:17 -0500 (Tue, 06 Mar 2007) $"
-__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__id__        = "$Id: speechgenerator.py 2343 2007-05-03 20:51:02Z richb $"
+__version__   = "$Revision: 2343 $"
+__date__      = "$Date: 2007-05-03 16:51:02 -0400 (Thu, 03 May 2007) $"
+__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import math
@@ -194,6 +194,9 @@ class SpeechGenerator:
         Returns a list of utterances to be spoken.
         """
         if obj.state.count(atspi.Accessibility.STATE_SENSITIVE) == 0:
+            # Translators: this represents an item on the screen that has
+            # been set insensitive (or grayed out).
+            #
             return [_("grayed")]
         else:
             return []
@@ -371,8 +374,12 @@ class SpeechGenerator:
 
         utterances = []
         if obj.state.count(atspi.Accessibility.STATE_CHECKED):
+            # Translators: this represents the state of a checkbox.
+            #
             checkedState = _("checked")
         else:
+            # Translators: this represents the state of a checkbox.
+            #
             checkedState = _("not checked")
 
         # If it's not already focused, say it's name
@@ -699,6 +706,9 @@ class SpeechGenerator:
                 hasItems = True
                 break
         if not hasItems:
+            # Translators: this is the number of items in a layered pane
+            # or table.
+            #
             utterances.append(_("0 items"))
 
         return utterances
@@ -785,6 +795,18 @@ class SpeechGenerator:
             utterances.extend(self._getSpeechForObjectAvailability(obj))
             utterances.extend(self._getSpeechForObjectAccelerator(obj))
 
+        # OpenOffice check menu items currently have a role of "menu item"
+        # rather then "check menu item", so we need to test if one of the
+        # states is CHECKED. If it is, then add that in to the list of
+        # speech utterances. Note that we can't tell if this is a "check
+        # menu item" that is currently unchecked and speak that state. 
+        # See Orca bug #433398 for more details.
+        #
+        if obj.state.count(atspi.Accessibility.STATE_CHECKED):
+            # Translators: this represents the state of a checked menu item.
+            #
+            utterances.append(_("checked"))
+
         self._debugGenerator("_getSpeechForMenuItem",
                              obj,
                              already_focused,
@@ -897,8 +919,9 @@ class SpeechGenerator:
         Returns a list of utterances to be spoken for the object.
         """
 
-        percentage = ("%d" % obj.value.currentValue) + " " \
-                     + _("percent") + ". "
+        # Translators: this is the percentage value of a progress bar.
+        #
+        percentage = _("%d percent.") % obj.value.currentValue + " "
 
         utterances = []
 
@@ -951,8 +974,14 @@ class SpeechGenerator:
 
         utterances = []
         if obj.state.count(atspi.Accessibility.STATE_CHECKED):
+            # Translators: this is in reference to a radio button being
+            # selected or not.
+            #
             selectionState = _("selected")
         else:
+            # Translators: this is in reference to a radio button being
+            # selected or not.
+            #
             selectionState = _("not selected")
 
         # If it's not already focused, say it's name
@@ -1185,6 +1214,9 @@ class SpeechGenerator:
                 hasItems = True
                 break
         if not hasItems:
+            # Translators: this is the number of items in a layered pane
+            # or table.
+            #
             utterances.append(_("0 items"))
 
         return utterances
@@ -1227,14 +1259,25 @@ class SpeechGenerator:
         #
         if obj.state.count(atspi.Accessibility.STATE_EXPANDABLE):
             if obj.state.count(atspi.Accessibility.STATE_EXPANDED):
+                # Translators: this represents the state of a node in a tree.
+                # 'expanded' means the children are showing.
+                # 'collapsed' means the children are not showing.
+                #
                 utterances.append(_("expanded"))
             else:
+                # Translators: this represents the state of a node in a tree.
+                # 'expanded' means the children are showing.
+                # 'collapsed' means the children are not showing.
+                #
                 utterances.append(_("collapsed"))
 
             # If this is an expandable table cell with no children, then
             # let the user know.
             #
             if obj.childCount == 0:
+                # Translators: this is the number of items in a layered pane
+                # or table.
+                #
                 utterances.append(_("0 items"))
 
         self._debugGenerator("_getSpeechForTableCell",
@@ -1349,8 +1392,11 @@ class SpeechGenerator:
         """
 
         if settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
-            utterances = [_("tear off menu item")]
+            utterances = [rolenames.getSpeechForRoleName(obj)]
         else:
+            # Translators: brief spoken words for the rolename of a tear off
+            # menu item.
+            #
             utterances = [_("tear off")]
 
         self._debugGenerator("_getSpeechForTearOffMenuItem",
@@ -1399,8 +1445,12 @@ class SpeechGenerator:
 
         utterances = []
         if obj.state.count(atspi.Accessibility.STATE_CHECKED):
+            # Translators: the state of a toggle button.
+            #
             checkedState = _("pressed")
         else:
+            # Translators: the state of a toggle button.
+            #
             checkedState = _("not pressed")
 
         # If it's not already focused, say it's name
@@ -1541,17 +1591,6 @@ class SpeechGenerator:
         if obj is stopAncestor:
             return utterances
 
-        # We try to ignore fillers and panels without names.
-        # [[[TODO: WDW - HACK sometimes table cells can be children
-        # of table cells (see the "Browse" dialog of gnome-terminal
-        # via "Edit" -> "Current Profile" -> "General" Tab ->
-        # "Profile icon:" button -> "Browse..." button - each element
-        # in the list is a compound table cell where the icon and
-        # text are child table cells of the table cell).  So...
-        # we happily ignore those as well.  One thing we might
-        # want to do is treat the parent as a compound object.
-        # Logged as bugzilla bug 319751.]]]
-        #
         parent = obj.parent
         if parent \
             and (obj.role == rolenames.ROLE_TABLE_CELL) \
@@ -1561,45 +1600,21 @@ class SpeechGenerator:
         while parent and (parent.parent != parent):
             if parent == stopAncestor:
                 break
-            if (parent.role != rolenames.ROLE_FILLER) \
-                and (parent.role != rolenames.ROLE_SECTION) \
-                and (parent.role != rolenames.ROLE_LAYERED_PANE) \
-                and (parent.role != rolenames.ROLE_SPLIT_PANE) \
-                and (parent.role != rolenames.ROLE_SCROLL_PANE) \
-                and (parent.role != rolenames.ROLE_UNKNOWN) \
-                and (not self._script.isLayoutOnly(parent)):
-
-                # Announce the label and text of the object in the hierarchy.
-                #
-                text = self._script.getDisplayedText(parent)
-                label = self._script.getDisplayedLabel(parent)
-
-                # Push announcement of cell after text and label.
-                #
-                if parent.role != rolenames.ROLE_TABLE_CELL:
-                    utterances.append(rolenames.getSpeechForRoleName(parent))
-
-                if text and len(text):
+            if not self._script.isLayoutOnly(parent):
+                text = self._script.getDisplayedLabel(parent)
+                if not text and parent.text:
+                    text = self._script.getDisplayedText(parent)
+                if text and len(text.strip()):
+                    # Push announcement of cell to the end
+                    #
+                    if not parent.role in [rolenames.ROLE_TABLE_CELL,
+                                           rolenames.ROLE_FILLER]:
+                        utterances.append(\
+                            rolenames.getSpeechForRoleName(parent))
                     utterances.append(text)
-                if label and len(label):
-                    utterances.append(label)
-
-                if parent.role == rolenames.ROLE_TABLE_CELL:
-                    utterances.append(rolenames.getSpeechForRoleName(parent))
-
-            # [[[TODO: HACK - we've discovered oddness in hierarchies
-            # such as the gedit Edit->Preferences dialog.  In this
-            # dialog, we have labeled groupings of objects.  The
-            # grouping is done via a FILLER with two children - one
-            # child is the overall label, and the other is the
-            # container for the grouped objects.  When we detect this,
-            # we add the label to the overall context.]]]
-            #
-            if parent.role == rolenames.ROLE_FILLER:
-                label = self._script.getDisplayedLabel(parent)
-                if label and len (label):
-                    utterances.append(label)
-
+                    if parent.role == rolenames.ROLE_TABLE_CELL:
+                        utterances.append(\
+                            rolenames.getSpeechForRoleName(parent))
             parent = parent.parent
 
         utterances.reverse()
